package ui;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.*;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.event.*;

import controller.ApplicationController;
import filter.*;

import model.*;


public class ApplicationView extends BambooView {
	ApplicationController controller;

//	JFileChooser fileChooser = new JFileChooser();
	JFileChooser fileChooser;

	JpegWatermarkFilter jpegWatermarkFilter = new JpegWatermarkFilter();

	PngFilter pngFilter = new PngFilter();

	AtfFilter atfFilter = new AtfFilter();

	private JpegFilter jpegFilter = new JpegFilter();

	private Graphics2D g = null;

	public ApplicationView(final ApplicationController controller,
			final BambooModel model) {
		super(controller, model);
		this.controller = controller;

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (UnsupportedLookAndFeelException e) {
			e.printStackTrace();
		}

		fileChooser = new JFileChooser();
		fileChooser.setAcceptAllFileFilterUsed(false);
//		fileChooser.seta
	}

	public BufferedImage createSaveImage() {
		if (controller.isRegion()) {
			Dimension d = controller.getSaveRegion();
			BufferedImage image = createImage(d);
			controller.write(g);
			return createSaveImage(image, d);
		} else {
			Dimension d = controller.getSaveSquare();
			BufferedImage image = createImage(d);
			controller.write(g);
			return createSaveImage(image, d);
		}

	}

	public void saveImage(File file, String suffix) {
		BufferedImage saveImage = createSaveImage();
		try {
			ImageIO.write(saveImage, suffix, file);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	protected BufferedImage createImage(Dimension d) {
		return createImage(d.width, d.height);
	}

	protected BufferedImage createImage(int width, int height) {
		BufferedImage image = new BufferedImage(width, height,
				BufferedImage.TYPE_INT_ARGB);

		g = image.createGraphics();

		g.setColor(Color.WHITE);
		g.fillRect(0, 0, width, height);

		return image;
	}

	protected BufferedImage createSaveImage(BufferedImage image, int width,
			int height) {

		g.dispose();

		BufferedImage saveImage = new BufferedImage(width + indent * 2, height
				+ indent * 2, BufferedImage.TYPE_INT_RGB);
		g = saveImage.createGraphics();
		g.fillRect(0, 0, width + indent * 2, height + indent * 2);
		g.drawImage(image, indent, indent, null);
		g.setColor(Color.BLACK);
		g.drawRect(indent - 1, indent - 1, width + 1, height + 1);

		return saveImage;
	}

	protected BufferedImage createSaveImage(BufferedImage image, Dimension d) {
		return createSaveImage(image, d.width, d.height);
	}

	@Override
	protected JPanel northPanel() {
		JPanel upperPanel = new JPanel(new BorderLayout());
		JToolBar toolBar = new JToolBar();
		toolBar.setFloatable(false);
		toolBar.setFocusable(false);
		JButton newButton = new JButton(new ImageIcon(getClass().getResource(
				"new.gif")));
		newButton.setToolTipText("New");
		newButton.addActionListener(new NewListener());
		newButton.setFocusable(false);
		toolBar.add(newButton);

		JButton openButton = new JButton(new ImageIcon(getClass().getResource(
				"open.gif")));
		openButton.setToolTipText("Open");
		openButton.setFocusable(false);
		openButton.addActionListener(new OpenListener());
		toolBar.add(openButton);

		JButton saveImageButton = new JButton(new ImageIcon(getClass()
				.getResource("save.gif")));
		saveImageButton.setToolTipText("Save");
		saveImageButton.setFocusable(false);
		saveImageButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				((ApplicationController) controller).save();
			}
		});
		toolBar.add(saveImageButton);

		JButton urlButton = new JButton("URL(Long)");
		urlButton.setToolTipText("URL");
		urlButton.setFocusable(false);
		urlButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				controller.url();
			}
		});
		toolBar.add(urlButton);

		JButton urlButton3 = new JButton("URL(Short)");
		urlButton3.setToolTipText("URL");
		urlButton3.setFocusable(false);
		urlButton3.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				controller.url3();
			}
		});
		toolBar.add(urlButton3);

		upperPanel.add(toolBar, BorderLayout.WEST);
		upperPanel.add(new LockButtonPanel(controller));
		upperPanel.add(sizeLabel(), BorderLayout.EAST);
		return upperPanel;
	}

	@Override
	public JPanel southPanel(){
//		StatusBar statusBar = new StatusBar();
		JPanel statusBar = new JPanel();
		statusBar.setLayout(new BorderLayout());
		JLabel coordinateLabel = new CoordinateLabel(controller);
		coordinateLabel.setHorizontalAlignment(SwingConstants.CENTER);
		statusBar.add(coordinateLabel, BorderLayout.CENTER);
		JPanel buttonPanel = new JPanel();
		buttonPanel.add(leftButton());
		buttonPanel.add(new LayerPlusButton(controller));
		buttonPanel.add(new LayerLabel(controller));
		buttonPanel.add(new ReverseButton(controller));

		JButton rotateButton = new RotateButton(controller);
		buttonPanel.add(rotateButton);
		if (controller.getRowCount() != controller.getColumnCount())
			rotateButton.setEnabled(false);
		statusBar.add(buttonPanel, BorderLayout.EAST);

		return statusBar;
	}
	class NewListener implements ActionListener {
		private JDialog dialog;

		private JSpinner spinner;

		private JSpinner sizeSpinner;

		int masu = 20;

		public void actionPerformed(ActionEvent e) {
			dialog = new JDialog(getFrame(), "New", true);
			JPanel panel = new JPanel(new GridLayout(2, 2));

			JLabel masuLabel = new JLabel("Number of Grid");
			panel.add(masuLabel);

			final SpinnerNumberModel masuModel = new SpinnerNumberModel(20, 1,
					1000, 1);
			spinner = new JSpinner(masuModel);
			final SpinnerNumberModel sizeModel = new SpinnerNumberModel(24, 1,
					24, 1);
			sizeSpinner = new JSpinner(sizeModel);
			spinner.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					masu = masuModel.getNumber().intValue();
					if (masu <= 25) {
						sizeModel.setValue(new Integer(24));
					} else {
						int size = 600 / masu;
						sizeModel.setValue(new Integer(size));
					}

				}
			});
			panel.add(spinner);

			JLabel sizeLabel = new JLabel("Size of Grid");
			panel.add(sizeLabel);
			panel.add(sizeSpinner);

			dialog.add(panel);

			JButton button = new JButton("OK");
			button.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					int size = sizeModel.getNumber().intValue();
					disposeFrame();
					controller.newFrame(masu, size);
				}
			});
			dialog.add(button, BorderLayout.SOUTH);
			dialog.pack();
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
			dialog.setLocation((screenSize.width - dialog.getWidth()) / 2,
					(screenSize.height - dialog.getHeight()) / 2); // �\���ʒu�𒲐�
			dialog.setVisible(true);
		}
	}

	class OpenListener implements ActionListener {
		public void actionPerformed(ActionEvent ae) {
/*			
			try {
				UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (UnsupportedLookAndFeelException e) {
				e.printStackTrace();
			}
*/
			fileChooser.resetChoosableFileFilters();
			fileChooser.addChoosableFileFilter(jpegWatermarkFilter);
			fileChooser.addChoosableFileFilter(pngFilter);
			fileChooser.addChoosableFileFilter(atfFilter);
			fileChooser.setFileFilter(jpegWatermarkFilter);
			
			fileChooser.setDialogTitle("Read Data *.jpeg *png *.atf");
			int returnVal = fileChooser.showOpenDialog(getFrame());

			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File file = fileChooser.getSelectedFile();
				if (!file.exists()) {
					return;
				}
				String fileName = file.getPath();
				
				String[] format = { "png", "jpeg", "atf" };
				if (!isAllowedFormat(fileName, format)) {
					return;
				}

				if (endsWithIgnoreCase(fileName, "atf")) {
					controller.loadAtf(file);
				} else if (endsWithIgnoreCase(fileName, "jpeg")
						|| endsWithIgnoreCase(fileName, "png")) {
					disposeFrame();
					controller.decode(file);
				}
			}
		}
	}

	public String save() {
		fileChooser.resetChoosableFileFilters();

		fileChooser.setDialogTitle("Save");
		fileChooser.addChoosableFileFilter(jpegWatermarkFilter);
		fileChooser.addChoosableFileFilter(pngFilter);
		fileChooser.addChoosableFileFilter(jpegFilter);
		fileChooser.addChoosableFileFilter(atfFilter);
		fileChooser.setFileFilter(jpegWatermarkFilter);

		int returnVal = fileChooser.showSaveDialog(getFrame());
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			File file = fileChooser.getSelectedFile();
			String fileName = file.getPath();

			String[] format = { "png", "jpg", "jpeg", "atf"/*, "dat" */};
			if (isAllowedFormat(fileName, format))
				return fileName;

			if (fileChooser.getFileFilter() instanceof JpegFilter) {
				fileName += ".jpg";
				return fileName;
			} else if (fileChooser.getFileFilter() instanceof AtfFilter) {
				fileName += ".atf";
				return fileName;
			} else if (fileChooser.getFileFilter() instanceof JpegWatermarkFilter) {
				fileName += ".jpeg";
				return fileName;
			} else if (fileChooser.getFileFilter() instanceof PngFilter) {
				fileName += ".png";
				return fileName;
			}
		}
		return null;
	}

	protected boolean isAllowedFormat(String fileName, String[] suffix) {
		String lowerName = fileName.toLowerCase();
		for (int i = 0; i < suffix.length; i++) {
			if (lowerName.endsWith("." + suffix[i]))
				return true;
		}

		return false;
	}

	protected boolean endsWithIgnoreCase(String str1, String suffix) {
		String lowerName = str1.toLowerCase();
		if (lowerName.endsWith("." + suffix)) {
			return true;
		}
		return false;
	}

	@Override
	protected void processKey(KeyEvent e) {
		super.processKey(e);
		int keyCode = e.getKeyCode();
		switch (keyCode) {
		case KeyEvent.VK_S:
			if (e.isControlDown()) {
				controller.save();
			} else {
				controller.setBamboo("horizontal");
			}
			break;
		}
	}
}
