package ui;

import img.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

import javax.swing.*;

import controller.BambooController;


@SuppressWarnings("serial")
public class BambooPanel extends JPanel {
	BambooController controller;
	private ImageMaker imageMaker;
	private Graphics2D graphics;
	private BufferedImage mapImage;

	public BambooPanel(final BambooController controller) {
		this.controller = controller;
		makeMouseListener();
		imageMaker = new ImageMaker(controller);
		mapImage = new BufferedImage(controller.getPanelWidth(), controller
				.getPanelHeight(), BufferedImage.TYPE_INT_BGR);
		graphics = (Graphics2D) mapImage.getGraphics();
		graphics.setColor(Color.WHITE);
		graphics.fillRect(0, 0, controller.getPanelWidth(), controller
				.getPanelHeight());
	}

	@Override
	public void paintComponent(final Graphics g) {
		super.paintComponent(g);
		imageMaker.makeMap(graphics);
		g.drawImage(mapImage, 0, 0, null);
	}

	private void makeMouseListener() {
		addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(final MouseEvent e) {
				controller.mousePressed(e.getX(), e.getY());
//				repaint();
			}
		});

		addMouseMotionListener(new MouseMotionAdapter() {
			@Override
			public void mouseDragged(final MouseEvent e) {
				controller.mouseDragged(e.getX(), e.getY());
//				repaint();
			}
		});
	}

	public void write(final Graphics2D g, final int left, final int right,
			final int top, final int bottom) {
		imageMaker.write(g, left, right, top, bottom);
	}

	public void newMap(final int size) {
		final int length = size * controller.getSizeOfCell() + 1;
		setPreferredSize(new Dimension(length, length));
		mapImage = new BufferedImage(length, length, BufferedImage.TYPE_INT_BGR);
		graphics = (Graphics2D) mapImage.getGraphics();
		graphics.setColor(Color.WHITE);
		graphics.fillRect(0, 0, length, length);
	}
}
